"""
Test module for transformations.
"""

from geojson import (
    Feature,
    FeatureCollection,
    LineString,
    MultiLineString,
    Point,
    Polygon,
)

from turfpy.transformation import (
    bbox_clip,
    bezier_spline,
    circle,
    concave,
    convex,
    difference,
    dissolve,
    intersect,
    line_offset,
    tesselate,
    transform_rotate,
    transform_scale,
    transform_translate,
    union,
    voronoi,
)


def test_circle():
    center = Feature(geometry=Point((-75.343, 39.984)))
    cc = circle(center, radius=5, steps=10)
    cc = cc["geometry"]
    assert cc.type == "Polygon"
    assert len(cc.coordinates[0]) == 11
    assert cc.coordinates == [
        [
            [-75.343, 40.028966],
            [-75.377513, 40.020373],
            [-75.398824, 39.997882],
            [-75.398802, 39.970091],
            [-75.377476, 39.947617],
            [-75.343, 39.939034],
            [-75.308524, 39.947617],
            [-75.287198, 39.970091],
            [-75.287176, 39.997882],
            [-75.308487, 40.020373],
            [-75.343, 40.028966],
        ]
    ]


def test_bbox_clip():
    f = Feature(
        geometry={
            "coordinates": [[[2, 2], [8, 4], [12, 8], [3, 7], [2, 2]]],
            "type": "Polygon",
        }
    )
    bbox = [0, 0, 10, 10]
    clip = bbox_clip(f, bbox)
    clip = clip["geometry"]
    assert clip.type == "Polygon"
    assert len(clip.coordinates[0]) == 6
    assert clip.coordinates == [
        [[10.0, 6.0], [10.0, 7.777778], [3.0, 7.0], [2.0, 2.0], [8.0, 4.0], [10.0, 6.0]]
    ]


def test_intersection():
    f = Feature(
        geometry={
            "coordinates": [
                [
                    [-122.801742, 45.48565],
                    [-122.801742, 45.60491],
                    [-122.584762, 45.60491],
                    [-122.584762, 45.48565],
                    [-122.801742, 45.48565],
                ]
            ],
            "type": "Polygon",
        }
    )
    b = Feature(
        geometry={
            "coordinates": [
                [
                    [-122.520217, 45.535693],
                    [-122.64038, 45.553967],
                    [-122.720031, 45.526554],
                    [-122.669906, 45.507309],
                    [-122.723464, 45.446643],
                    [-122.532577, 45.408574],
                    [-122.487258, 45.477466],
                    [-122.520217, 45.535693],
                ]
            ],
            "type": "Polygon",
        }
    )
    inter = intersect([f, b])
    inter = inter["geometry"]
    assert inter.type == "Polygon"
    assert len(inter.coordinates[0]) == 7
    assert inter.coordinates == [
        [
            [-122.669906, 45.507309],
            [-122.689027, 45.48565],
            [-122.584762, 45.48565],
            [-122.584762, 45.545509],
            [-122.64038, 45.553967],
            [-122.720031, 45.526554],
            [-122.669906, 45.507309],
        ]
    ]

    a = Feature(
        geometry={
            "coordinates": [
                [
                    [0, 0],
                    [0, 3],
                    [3, 0],
                    [0, 0],
                ]
            ],
            "type": "Polygon",
        }
    )
    b = Feature(
        geometry={
            "coordinates": [
                [
                    [0, 3],
                    [1, 2],
                    [3, 3],
                    [1.8, 1],
                    [3, 0],
                    [7, 0],
                    [0, 7],
                    [0, 3],
                ]
            ],
            "type": "Polygon",
        }
    )
    inter = intersect([a, b])
    inter = inter.geometry
    assert inter.type == "GeometryCollection"
    assert len(inter.geometries) == 2
    assert inter.geometries[0]["type"] == "Polygon"
    assert inter.geometries[1]["type"] == "LineString"


def test_bezier_spline():
    ls = LineString(
        [
            (-76.091308, 18.427501),
            (-76.695556, 18.729501),
            (-76.552734, 19.40443),
            (-74.61914, 19.134789),
            (-73.652343, 20.07657),
            (-73.157958, 20.210656),
        ]
    )

    f = Feature(geometry=ls)

    bf = bezier_spline(f)
    bf = bf["geometry"]
    assert bf.type == "LineString"
    assert len(bf.coordinates) == 500


def test_union():
    poly1 = Feature(
        geometry={
            "type": "Polygon",
            "coordinates": [
                [
                    [-82.574787, 35.594087],
                    [-82.574787, 35.615581],
                    [-82.545261, 35.615581],
                    [-82.545261, 35.594087],
                    [-82.574787, 35.594087],
                ]
            ],
        }
    )

    poly2 = Feature(
        geometry={
            "type": "Polygon",
            "coordinates": [
                [
                    [-82.560024, 35.585153],
                    [-82.560024, 35.602602],
                    [-82.52964, 35.602602],
                    [-82.52964, 35.585153],
                    [-82.560024, 35.585153],
                ]
            ],
        }
    )
    result = union([poly1, poly2])
    assert dict(result) == {
        "type": "Feature",
        "geometry": {
            "coordinates": [
                [
                    [-82.560024, 35.594087],
                    [-82.574787, 35.594087],
                    [-82.574787, 35.615581],
                    [-82.545261, 35.615581],
                    [-82.545261, 35.602602],
                    [-82.52964, 35.602602],
                    [-82.52964, 35.585153],
                    [-82.560024, 35.585153],
                    [-82.560024, 35.594087],
                ]
            ],
            "type": "Polygon",
        },
        "properties": {},
    }

    p1 = Feature(geometry={"type": "Point", "coordinates": [-82.574787, 35.594087]})
    result2 = union([poly1, p1])
    assert result2 == poly1


def test_concave():
    f1 = Feature(geometry=Point((-63.601226, 44.642643)))
    f2 = Feature(geometry=Point((-63.591442, 44.651436)))
    f3 = Feature(geometry=Point((-63.580799, 44.648749)))
    f4 = Feature(geometry=Point((-63.573589, 44.641788)))
    f5 = Feature(geometry=Point((-63.587665, 44.64533)))
    f6 = Feature(geometry=Point((-63.595218, 44.64765)))
    fc = [f1, f2, f3, f4, f5, f6]
    concave_hull = concave(FeatureCollection(fc), alpha=100)

    assert concave_hull["type"] == "Feature"
    assert len(concave_hull["geometry"]["coordinates"][0]) == 7


def test_convex():
    f1 = Feature(geometry=Point((10.195312, 43.755225)))
    f2 = Feature(geometry=Point((10.404052, 43.8424511)))
    f3 = Feature(geometry=Point((10.579833, 43.659924)))
    f4 = Feature(geometry=Point((10.360107, 43.516688)))
    f5 = Feature(geometry=Point((10.14038, 43.588348)))
    f6 = Feature(geometry=Point((10.195312, 43.755225)))
    fc = [f1, f2, f3, f4, f5, f6]
    convex_hull = convex(FeatureCollection(fc))

    assert convex_hull["type"] == "Feature"
    assert len(convex_hull["geometry"]["coordinates"][0]) == 6


def test_dissolve():
    f1 = Feature(
        geometry=Polygon([[[0, 0], [0, 1], [1, 1], [1, 0], [0, 0]]]),
        properties={"combine": "yes", "fill": "#00f"},
    )

    f2 = Feature(
        geometry=Polygon([[[0, -1], [0, 0], [1, 0], [1, -1], [0, -1]]]),
        properties={"combine": "yes"},
    )

    f3 = Feature(
        geometry=Polygon([[[1, -1], [1, 0], [2, 0], [2, -1], [1, -1]]]),
        properties={"combine": "no"},
    )

    dissolve_result = dissolve([f1, f2, f3], property_name="combine")

    assert dissolve_result["type"] == "FeatureCollection"
    assert len(dissolve_result["features"]) == 2
    assert dissolve_result[0]["properties"] == {"combine": "yes", "fill": "#00f"}
    assert dissolve_result[1]["properties"] == {"combine": "no"}


def test_difference():
    f1 = Feature(
        geometry=Polygon([[[128, -26], [141, -26], [141, -21], [128, -21], [128, -26]]]),
        properties={"combine": "yes", "fill": "#00f"},
    )

    f2 = Feature(
        geometry=Polygon([[[126, -28], [140, -28], [140, -20], [126, -20], [126, -28]]]),
        properties={"combine": "yes"},
    )

    difference_result = difference(f1, f2)
    assert difference_result["type"] == "Feature"
    assert len(difference_result["geometry"]["coordinates"][0]) == 5
    assert difference_result["geometry"]["coordinates"][0] == [
        [141.0, -21.0],
        [141.0, -26.0],
        [140.0, -26.0],
        [140.0, -21.0],
        [141.0, -21.0],
    ]


def test_transform_rotate():
    f = Feature(geometry=Polygon([[[0, 29], [3.5, 29], [2.5, 32], [0, 29]]]))
    pivot = [0, 25]
    rotated_feature = transform_rotate(f, 10, pivot)

    assert rotated_feature["type"] == "Feature"
    assert len(rotated_feature["geometry"]["coordinates"][0]) == 4

    assert rotated_feature["geometry"]["coordinates"][0] == [
        [0.779582, 28.939231],
        [4.215029, 28.397872],
        [3.837175, 31.512519],
        [0.779582, 28.939231],
    ]


def test_transform_translate():

    f = Feature(geometry=Polygon([[[0, 29], [3.5, 29], [2.5, 32], [0, 29]]]))

    translate_feature = transform_translate(f, 100, 35, mutate=True)

    assert translate_feature["type"] == "Feature"
    assert len(translate_feature["geometry"]["coordinates"][0]) == 4
    assert translate_feature["geometry"]["coordinates"][0] == [
        [0.591903, 29.73668],
        [4.091903, 29.73668],
        [3.110728, 32.73668],
        [0.591903, 29.73668],
    ]


def test_transform_scale():
    f = Feature(geometry=Polygon([[[0, 29], [3.5, 29], [2.5, 32], [0, 29]]]))

    translate_feature = transform_scale(f, 3, origin=[0, 29])

    assert translate_feature["type"] == "Feature"
    assert len(translate_feature["geometry"]["coordinates"][0]) == 4
    assert translate_feature["geometry"]["coordinates"][0] == [
        [0.0, 29.0],
        [10.5, 29.0],
        [7.763024, 38.0],
        [0.0, 29.0],
    ]


def test_tesselate():
    f = Feature(
        geometry={
            "coordinates": [
                [[11, 0], [22, 4], [31, 0], [31, 11], [21, 15], [11, 11], [11, 0]]
            ],
            "type": "Polygon",
        }
    )
    result = tesselate(f)
    assert result == {
        "type": "FeatureCollection",
        "features": [
            {
                "geometry": {
                    "coordinates": [[[11, 11], [11, 0], [22, 4], [11, 11]]],
                    "type": "Polygon",
                },
                "properties": {},
                "type": "Feature",
            },
            {
                "geometry": {
                    "coordinates": [[[22, 4], [31, 0], [31, 11], [22, 4]]],
                    "type": "Polygon",
                },
                "properties": {},
                "type": "Feature",
            },
            {
                "geometry": {
                    "coordinates": [[[31, 11], [21, 15], [11, 11], [31, 11]]],
                    "type": "Polygon",
                },
                "properties": {},
                "type": "Feature",
            },
            {
                "geometry": {
                    "coordinates": [[[11, 11], [22, 4], [31, 11], [11, 11]]],
                    "type": "Polygon",
                },
                "properties": {},
                "type": "Feature",
            },
        ],
    }


def test_line_offset_multilinestring():
    ls = Feature(
        geometry=MultiLineString(
            [
                [(3.75, 9.25), (-130.95, 1.52)],
                [(23.15, -34.25), (-1.35, -4.65), (3.45, 77.95)],
            ]
        )
    )

    lined_off = line_offset(ls, 2, unit="mi")

    assert lined_off["type"] == "Feature"
    assert len(lined_off["geometry"]["coordinates"]) == 2
    assert lined_off["geometry"]["coordinates"] == [
        [[3.748342, 9.278899], [-130.951658, 1.548899]],
        [[23.172299, -34.231543], [-1.320442, -4.640314], [3.478898, 77.948321]],
    ]


def test_line_offset_linestring():
    ls = Feature(
        geometry=LineString(
            [
                [-70.3974009, -23.6494371],
                [-70.3973829, -23.6494227],
                [-70.3965343, -23.6487429],
            ]
        )
    )

    lined_off_positive = line_offset(ls, 0.01, unit="km")
    lined_off_negative = line_offset(ls, -0.01, unit="km")

    assert lined_off_positive["type"] == "Feature"
    assert lined_off_negative["type"] == "Feature"
    assert len(lined_off_positive["geometry"]["coordinates"]) == len(
        ls["geometry"]["coordinates"]
    )
    assert len(lined_off_negative["geometry"]["coordinates"]) == len(
        ls["geometry"]["coordinates"]
    )
    assert lined_off_positive["geometry"]["coordinates"] == [
        [-70.397346, -23.649508],
        [-70.397327, -23.649494],
        [-70.396478, -23.648813],
    ]
    assert lined_off_negative["geometry"]["coordinates"] == [
        [-70.397456, -23.649366],
        [-70.397439, -23.649352],
        [-70.39659, -23.648673],
    ]


def test_voronoi():
    """Test Voronoi."""
    points = [
        [-66.97035175371198, 40.318345269995234],
        [-63.776304623759685, 40.45001590163247],
        [-65.41960483851452, 42.13985310302137],
        [-69.58133775268497, 43.95405461286195],
        [-65.66337553550034, 55.97088945355232],
        [-60.280418548905, 56.240669185466146],
        [-68.5129561347689, 50.12984589640148],
        [-64.2393519226657, 59.66235385923687],
        [-60.90340080176197, 47.319854339485836],
        [-64.013392235184, 47.48505628754743],
        [-63.5901237814622, 56.435271304519375],
        [-65.29273842230272, 40.46532951990474],
        [-64.34074762923518, 44.41467903845307],
        [-62.75238118105902, 50.232555998274044],
        [-66.39216606663494, 42.05774847944996],
        [-69.4747201276131, 43.288490094187104],
        [-60.22111554327566, 53.23070074089705],
        [-68.38456573407701, 58.12528229586756],
        [-60.022015858921954, 57.16751878858943],
        [-67.27241103543064, 46.52814784233641],
        [-68.44640005497628, 49.31604545534581],
        [-63.19928242836677, 40.397501696572654],
        [-69.75867935840617, 45.47856756289996],
        [-61.32766380696367, 58.672721168560955],
        [-67.52671607410451, 55.62959559068972],
        [-64.05030735753337, 56.19827268873002],
        [-65.20026060312107, 52.91890866887687],
        [-63.02174849203104, 45.34033157319018],
        [-62.33913739166953, 43.46950736683852],
        [-68.70183827840249, 51.16845764645581],
        [-66.20557198738521, 45.61818662858589],
        [-67.06923935761512, 56.49552328077417],
        [-68.45745370027925, 53.01501887198211],
        [-61.230713576641044, 56.183690218293776],
        [-61.366266080040376, 45.09667176473487],
        [-68.40160283717302, 51.22288379742737],
        [-65.91059565227219, 46.12828610239469],
        [-69.24685336802877, 58.04159904298512],
        [-67.90746049167338, 51.458754170538626],
        [-64.69458939381633, 43.50862977964408],
        [-69.67707467644576, 55.13014562401008],
        [-67.86563100693668, 44.71454319618893],
        [-67.64200780701925, 45.346156738904824],
        [-62.784041537868234, 44.021481858972024],
        [-69.46030604589015, 50.77927933384373],
        [-67.47301069491814, 57.20749222445842],
        [-63.63826397952049, 43.06756343494853],
        [-65.03421426631598, 58.607691301001665],
        [-61.42694866295021, 44.01728814041832],
        [-65.27780516075711, 49.1577380024088],
        [-63.22809792736968, 54.75459925926561],
        [-65.75164600671681, 50.444095477144174],
        [-66.53391928446148, 46.81946313004039],
        [-61.17532370859516, 56.490308715502884],
        [-62.82596239604844, 47.62933849172895],
        [-67.91427680286654, 50.002068640008446],
        [-68.8281390917315, 41.624594548778255],
        [-62.324565557935586, 54.36070911572615],
        [-67.79789775299722, 54.032762733269294],
        [-69.90721635010229, 42.53005065790635],
        [-64.09218727436058, 44.21747980258202],
        [-65.45745182285232, 41.92888759186806],
        [-64.89743431251524, 56.02354218555687],
        [-62.74330413423213, 49.851812304102474],
        [-69.01724834169485, 43.86098277913519],
        [-65.08442833608343, 49.05799844118643],
        [-60.117926604673414, 42.56576378184112],
        [-63.380264668516965, 58.718930212178506],
        [-69.58981710789907, 56.45741811079546],
        [-69.34434256360679, 55.54353852333105],
        [-68.96303460007613, 58.368441094579225],
        [-60.499978235077876, 58.04095533357255],
        [-64.48228884995504, 52.67791964667544],
        [-63.28389119770206, 51.760879575173384],
        [-67.75617507537125, 43.75912562117327],
        [-68.00927728902863, 45.671594734286614],
        [-63.36784409964047, 44.10569333994948],
        [-65.58756228301353, 54.97295500158505],
        [-67.37277146991484, 56.476728117598626],
        [-68.10830042153275, 55.72080967470162],
        [-69.12830174563588, 45.58632852197277],
        [-64.6315835789136, 51.88413518274077],
        [-67.75853090943711, 58.587494282518776],
        [-61.45846709902253, 53.270601672273],
        [-61.903530354456734, 57.98191726730208],
        [-65.81769750926455, 42.034979844034275],
        [-64.2364381676968, 51.762721823182545],
        [-69.75124251822547, 50.065087854580355],
        [-62.624081650070146, 49.81702630730683],
        [-64.44355690073627, 59.06794579492187],
        [-68.55204564864789, 55.336271363856184],
        [-63.25564507829077, 50.6046345112907],
        [-63.81456104951526, 40.8486627152392],
        [-64.37015352762566, 58.00983759616774],
        [-61.870172642975895, 59.15750997986467],
        [-60.02123672191712, 56.074643702400614],
        [-60.11304594655109, 43.241774996887635],
        [-66.18978478841018, 53.56622582843157],
        [-63.92923317441267, 54.78528722989218],
        [-62.48536066011403, 54.50280730402256],
    ]
    bbox = [-70, 40, -60, 60]
    result = voronoi(points, bbox)
    assert result["type"] == "Feature"
    assert result == {
        "geometry": {
            "coordinates": [
                [
                    [
                        [-69.165621, 44.71768],
                        [-68.778553, 44.742594],
                        [-68.357297, 44.174238],
                        [-68.456663, 42.944],
                        [-69.341595, 43.651138],
                        [-69.165621, 44.71768],
                    ]
                ],
                [
                    [
                        [-70.0, 43.545667],
                        [-70.0, 44.677924],
                        [-69.311112, 44.75806],
                        [-69.165621, 44.71768],
                        [-69.341595, 43.651138],
                        [-70.0, 43.545667],
                    ]
                ],
                [
                    [
                        [-70.0, 43.085494],
                        [-70.0, 43.545667],
                        [-69.341595, 43.651138],
                        [-68.456663, 42.944],
                        [-68.402835, 42.747442],
                        [-69.000144, 42.515331],
                        [-70.0, 43.085494],
                    ]
                ],
                [
                    [
                        [-70.0, 44.677924],
                        [-70.0, 47.704206],
                        [-69.803243, 47.636922],
                        [-69.311112, 44.75806],
                        [-70.0, 44.677924],
                    ]
                ],
                [
                    [
                        [-69.311112, 44.75806],
                        [-69.803243, 47.636922],
                        [-68.920065, 47.475451],
                        [-68.672282, 46.987185],
                        [-68.528813, 45.10431],
                        [-68.778553, 44.742594],
                        [-69.165621, 44.71768],
                        [-69.311112, 44.75806],
                    ]
                ],
                [
                    [
                        [-70.0, 48.120701],
                        [-70.0, 50.582777],
                        [-69.13914, 50.232093],
                        [-69.109818, 49.67141],
                        [-70.0, 48.120701],
                    ]
                ],
                [
                    [
                        [-70.0, 47.704206],
                        [-70.0, 48.120701],
                        [-69.109818, 49.67141],
                        [-68.2834, 49.738998],
                        [-66.890978, 48.658947],
                        [-66.897845, 48.521491],
                        [-67.461655, 48.08959],
                        [-68.920065, 47.475451],
                        [-69.803243, 47.636922],
                        [-70.0, 47.704206],
                    ]
                ],
                [
                    [
                        [-68.456663, 42.944],
                        [-68.357297, 44.174238],
                        [-66.60483, 44.375006],
                        [-66.192858, 44.031389],
                        [-66.229319, 43.585762],
                        [-67.709376, 42.399187],
                        [-68.402835, 42.747442],
                        [-68.456663, 42.944],
                    ]
                ],
                [
                    [
                        [-70.0, 50.582777],
                        [-70.0, 52.364149],
                        [-69.732985, 52.244378],
                        [-68.888249, 50.598076],
                        [-69.13914, 50.232093],
                        [-70.0, 50.582777],
                    ]
                ],
                [
                    [
                        [-69.13914, 50.232093],
                        [-68.888249, 50.598076],
                        [-68.457587, 50.676396],
                        [-68.091294, 50.63908],
                        [-68.2834, 49.738998],
                        [-69.109818, 49.67141],
                        [-69.13914, 50.232093],
                    ]
                ],
                [
                    [
                        [-61.907059, 58.570287],
                        [-60.62946, 60.0],
                        [-60.0, 60.0],
                        [-60.0, 59.554048],
                        [-61.200486, 57.981275],
                        [-61.907059, 58.570287],
                    ]
                ],
                [
                    [
                        [-61.200486, 57.981275],
                        [-61.176593, 57.41325],
                        [-62.390875, 56.820434],
                        [-63.047906, 57.536916],
                        [-62.523439, 58.587777],
                        [-61.907059, 58.570287],
                        [-61.200486, 57.981275],
                    ]
                ],
                [
                    [
                        [-62.523439, 58.587777],
                        [-63.047906, 57.536916],
                        [-63.298993, 57.55999],
                        [-64.019466, 58.565766],
                        [-63.816295, 59.184735],
                        [-62.930959, 59.990927],
                        [-62.523439, 58.587777],
                    ]
                ],
                [
                    [
                        [-60.904624, 51.240154],
                        [-60.793568, 54.684065],
                        [-60.755079, 54.697224],
                        [-60.0, 54.644156],
                        [-60.0, 50.522463],
                        [-60.328187, 50.753482],
                        [-60.904624, 51.240154],
                    ]
                ],
                [
                    [
                        [-60.755079, 54.697224],
                        [-60.793568, 54.684065],
                        [-60.798058, 54.684416],
                        [-61.702232, 55.226953],
                        [-62.461734, 55.793861],
                        [-62.464923, 55.798318],
                        [-62.384722, 56.550471],
                        [-60.758232, 56.25665],
                        [-60.700817, 55.29907],
                        [-60.755079, 54.697224],
                    ]
                ],
                [
                    [
                        [-60.758232, 56.25665],
                        [-60.59862, 56.828828],
                        [-60.0, 56.661935],
                        [-60.0, 56.393113],
                        [-60.700817, 55.29907],
                        [-60.758232, 56.25665],
                    ]
                ],
                [
                    [
                        [-62.03623, 46.408914],
                        [-61.679226, 48.626676],
                        [-60.0, 49.783749],
                        [-60.0, 45.971991],
                        [-62.02117, 46.392798],
                        [-62.03623, 46.408914],
                    ]
                ],
                [
                    [
                        [-60.328187, 50.753482],
                        [-62.622199, 50.04518],
                        [-63.291895, 50.029214],
                        [-62.423765, 51.203424],
                        [-61.540015, 51.510769],
                        [-60.904624, 51.240154],
                        [-60.328187, 50.753482],
                    ]
                ],
                [
                    [
                        [-60.904624, 51.240154],
                        [-61.540015, 51.510769],
                        [-62.829759, 53.070215],
                        [-60.798058, 54.684416],
                        [-60.793568, 54.684065],
                        [-60.904624, 51.240154],
                    ]
                ],
                [
                    [
                        [-63.000505, 48.748604],
                        [-62.622199, 50.04518],
                        [-60.328187, 50.753482],
                        [-60.0, 50.522463],
                        [-60.0, 49.783749],
                        [-61.679226, 48.626676],
                        [-63.000505, 48.748604],
                    ]
                ],
                [
                    [
                        [-64.533301, 40.578529],
                        [-64.713039, 41.271618],
                        [-64.340982, 41.837472],
                        [-64.21284, 41.996761],
                        [-62.593969, 41.868138],
                        [-63.464372, 40.68111],
                        [-64.533301, 40.578529],
                    ]
                ],
                [
                    [
                        [-64.539143, 40.0],
                        [-64.533301, 40.578529],
                        [-63.464372, 40.68111],
                        [-63.526359, 40.0],
                        [-64.539143, 40.0],
                    ]
                ],
                [
                    [
                        [-66.186908, 41.023722],
                        [-66.144297, 41.053141],
                        [-65.885626, 41.139652],
                        [-64.713039, 41.271618],
                        [-64.533301, 40.578529],
                        [-64.539143, 40.0],
                        [-66.097214, 40.0],
                        [-66.186908, 41.023722],
                    ]
                ],
                [
                    [
                        [-68.778553, 44.742594],
                        [-68.528813, 45.10431],
                        [-68.131254, 45.163981],
                        [-66.772422, 44.682886],
                        [-66.60483, 44.375006],
                        [-68.357297, 44.174238],
                        [-68.778553, 44.742594],
                    ]
                ],
                [
                    [
                        [-68.920065, 47.475451],
                        [-67.461655, 48.08959],
                        [-66.689239, 46.131497],
                        [-66.981801, 45.788497],
                        [-67.448546, 45.934443],
                        [-68.672282, 46.987185],
                        [-68.920065, 47.475451],
                    ]
                ],
                [
                    [
                        [-68.672282, 46.987185],
                        [-67.448546, 45.934443],
                        [-68.131254, 45.163981],
                        [-68.528813, 45.10431],
                        [-68.672282, 46.987185],
                    ]
                ],
                [
                    [
                        [-70.0, 52.364149],
                        [-70.0, 53.53475],
                        [-69.018096, 54.100934],
                        [-67.240581, 52.949003],
                        [-67.155688, 52.599753],
                        [-68.524546, 52.11599],
                        [-68.717465, 52.109978],
                        [-69.732985, 52.244378],
                        [-70.0, 52.364149],
                    ]
                ],
                [
                    [
                        [-69.732985, 52.244378],
                        [-68.717465, 52.109978],
                        [-68.457587, 50.676396],
                        [-68.888249, 50.598076],
                        [-69.732985, 52.244378],
                    ]
                ],
                [
                    [
                        [-70.0, 55.730663],
                        [-70.0, 55.857332],
                        [-68.804662, 56.178409],
                        [-68.787076, 56.055792],
                        [-69.068307, 54.980762],
                        [-70.0, 55.730663],
                    ]
                ],
                [
                    [
                        [-68.804662, 56.178409],
                        [-68.509799, 56.771456],
                        [-68.370847, 56.712079],
                        [-67.749682, 56.107668],
                        [-67.915551, 55.050079],
                        [-68.787076, 56.055792],
                        [-68.804662, 56.178409],
                    ]
                ],
                [
                    [
                        [-68.787076, 56.055792],
                        [-67.915551, 55.050079],
                        [-67.862662, 54.865204],
                        [-68.933613, 54.245603],
                        [-69.068307, 54.980762],
                        [-68.787076, 56.055792],
                    ]
                ],
                [
                    [
                        [-68.933613, 54.245603],
                        [-67.862662, 54.865204],
                        [-66.767766, 54.679264],
                        [-66.750431, 54.63851],
                        [-67.240581, 52.949003],
                        [-69.018096, 54.100934],
                        [-68.933613, 54.245603],
                    ]
                ],
                [
                    [
                        [-68.801977, 57.941933],
                        [-68.425974, 58.836434],
                        [-67.717239, 57.8765],
                        [-68.593112, 57.006576],
                        [-68.720108, 57.098347],
                        [-68.801977, 57.941933],
                    ]
                ],
                [
                    [
                        [-68.637582, 60.0],
                        [-66.406769, 60.0],
                        [-66.393058, 58.150471],
                        [-67.717239, 57.8765],
                        [-68.425974, 58.836434],
                        [-68.637582, 60.0],
                    ]
                ],
                [
                    [
                        [-60.758232, 56.25665],
                        [-62.384722, 56.550471],
                        [-62.390875, 56.820434],
                        [-61.176593, 57.41325],
                        [-60.861104, 57.275846],
                        [-60.59862, 56.828828],
                        [-60.758232, 56.25665],
                    ]
                ],
                [
                    [
                        [-62.384722, 56.550471],
                        [-62.464923, 55.798318],
                        [-63.444534, 55.587305],
                        [-64.224386, 57.101556],
                        [-63.298993, 57.55999],
                        [-63.047906, 57.536916],
                        [-62.390875, 56.820434],
                        [-62.384722, 56.550471],
                    ]
                ],
                [
                    [
                        [-64.224386, 57.101556],
                        [-64.662258, 57.024246],
                        [-65.363808, 57.210479],
                        [-65.572056, 57.342562],
                        [-64.500872, 58.53237],
                        [-64.019466, 58.565766],
                        [-63.298993, 57.55999],
                        [-64.224386, 57.101556],
                    ]
                ],
                [
                    [
                        [-63.444534, 55.587305],
                        [-63.545405, 55.529856],
                        [-64.339978, 55.461772],
                        [-64.662258, 57.024246],
                        [-64.224386, 57.101556],
                        [-63.444534, 55.587305],
                    ]
                ],
                [
                    [
                        [-62.464923, 55.798318],
                        [-62.461734, 55.793861],
                        [-63.246204, 53.479829],
                        [-63.291536, 53.375844],
                        [-63.630718, 53.58069],
                        [-63.545405, 55.529856],
                        [-63.444534, 55.587305],
                        [-62.464923, 55.798318],
                    ]
                ],
                [
                    [
                        [-67.717239, 57.8765],
                        [-66.393058, 58.150471],
                        [-66.046137, 57.546222],
                        [-67.244563, 56.866571],
                        [-68.370847, 56.712079],
                        [-68.509799, 56.771456],
                        [-68.593112, 57.006576],
                        [-67.717239, 57.8765],
                    ]
                ],
                [
                    [
                        [-66.406769, 60.0],
                        [-65.784478, 60.0],
                        [-65.445288, 59.744365],
                        [-64.500872, 58.53237],
                        [-65.572056, 57.342562],
                        [-65.921232, 57.425878],
                        [-66.046137, 57.546222],
                        [-66.393058, 58.150471],
                        [-66.406769, 60.0],
                    ]
                ],
                [
                    [
                        [-64.019466, 58.565766],
                        [-64.500872, 58.53237],
                        [-65.445288, 59.744365],
                        [-63.816295, 59.184735],
                        [-64.019466, 58.565766],
                    ]
                ],
                [
                    [
                        [-65.363808, 57.210479],
                        [-65.246275, 55.50073],
                        [-66.522429, 55.40378],
                        [-66.572933, 55.679512],
                        [-65.921232, 57.425878],
                        [-65.572056, 57.342562],
                        [-65.363808, 57.210479],
                    ]
                ],
                [
                    [
                        [-64.662258, 57.024246],
                        [-64.339978, 55.461772],
                        [-64.726679, 55.159408],
                        [-65.246275, 55.50073],
                        [-65.363808, 57.210479],
                        [-64.662258, 57.024246],
                    ]
                ],
                [
                    [
                        [-64.726679, 55.159408],
                        [-64.85245, 54.048027],
                        [-65.21253, 53.980132],
                        [-66.750431, 54.63851],
                        [-66.767766, 54.679264],
                        [-66.522429, 55.40378],
                        [-65.246275, 55.50073],
                        [-64.726679, 55.159408],
                    ]
                ],
                [
                    [
                        [-64.339978, 55.461772],
                        [-63.545405, 55.529856],
                        [-63.630718, 53.58069],
                        [-64.501956, 53.809337],
                        [-64.85245, 54.048027],
                        [-64.726679, 55.159408],
                        [-64.339978, 55.461772],
                    ]
                ],
                [
                    [
                        [-66.750431, 54.63851],
                        [-65.21253, 53.980132],
                        [-66.477256, 52.046803],
                        [-67.155688, 52.599753],
                        [-67.240581, 52.949003],
                        [-66.750431, 54.63851],
                    ]
                ],
                [
                    [
                        [-66.477256, 52.046803],
                        [-66.390077, 51.885168],
                        [-66.935723, 50.725848],
                        [-67.863057, 50.730188],
                        [-68.524546, 52.11599],
                        [-67.155688, 52.599753],
                        [-66.477256, 52.046803],
                    ]
                ],
                [
                    [
                        [-65.21253, 53.980132],
                        [-64.85245, 54.048027],
                        [-64.501956, 53.809337],
                        [-64.98774, 52.362053],
                        [-66.009697, 51.80042],
                        [-66.390077, 51.885168],
                        [-66.477256, 52.046803],
                        [-65.21253, 53.980132],
                    ]
                ],
                [
                    [
                        [-63.630718, 53.58069],
                        [-63.291536, 53.375844],
                        [-63.143452, 53.185966],
                        [-63.758966, 52.381606],
                        [-64.307805, 52.234171],
                        [-64.98774, 52.362053],
                        [-64.501956, 53.809337],
                        [-63.630718, 53.58069],
                    ]
                ],
                [
                    [
                        [-67.448546, 45.934443],
                        [-66.981801, 45.788497],
                        [-66.772422, 44.682886],
                        [-68.131254, 45.163981],
                        [-67.448546, 45.934443],
                    ]
                ],
                [
                    [
                        [-63.000505, 48.748604],
                        [-63.567003, 48.769673],
                        [-63.706995, 48.844798],
                        [-63.955472, 49.577609],
                        [-63.291895, 50.029214],
                        [-62.622199, 50.04518],
                        [-63.000505, 48.748604],
                    ]
                ],
                [
                    [
                        [-63.000505, 48.748604],
                        [-61.679226, 48.626676],
                        [-62.03623, 46.408914],
                        [-63.293215, 46.516428],
                        [-63.567003, 48.769673],
                        [-63.000505, 48.748604],
                    ]
                ],
                [
                    [
                        [-63.567003, 48.769673],
                        [-63.293215, 46.516428],
                        [-64.39133, 46.008699],
                        [-65.311425, 47.295289],
                        [-65.41372, 47.682668],
                        [-63.706995, 48.844798],
                        [-63.567003, 48.769673],
                    ]
                ],
                [
                    [
                        [-63.706995, 48.844798],
                        [-65.41372, 47.682668],
                        [-65.790402, 47.926576],
                        [-64.661564, 50.115186],
                        [-64.482861, 50.201208],
                        [-63.955472, 49.577609],
                        [-63.706995, 48.844798],
                    ]
                ],
                [
                    [
                        [-63.955472, 49.577609],
                        [-64.482861, 50.201208],
                        [-64.504725, 50.541144],
                        [-63.761308, 51.170749],
                        [-62.423765, 51.203424],
                        [-63.291895, 50.029214],
                        [-63.955472, 49.577609],
                    ]
                ],
                [
                    [
                        [-68.717465, 52.109978],
                        [-68.524546, 52.11599],
                        [-67.863057, 50.730188],
                        [-68.091294, 50.63908],
                        [-68.457587, 50.676396],
                        [-68.717465, 52.109978],
                    ]
                ],
                [
                    [
                        [-67.863057, 50.730188],
                        [-66.935723, 50.725848],
                        [-66.660415, 49.378892],
                        [-66.890978, 48.658947],
                        [-68.2834, 49.738998],
                        [-68.091294, 50.63908],
                        [-67.863057, 50.730188],
                    ]
                ],
                [
                    [
                        [-66.390077, 51.885168],
                        [-66.009697, 51.80042],
                        [-64.743651, 50.815689],
                        [-64.504725, 50.541144],
                        [-64.482861, 50.201208],
                        [-64.661564, 50.115186],
                        [-66.660415, 49.378892],
                        [-66.935723, 50.725848],
                        [-66.390077, 51.885168],
                    ]
                ],
                [
                    [
                        [-65.921232, 57.425878],
                        [-66.572933, 55.679512],
                        [-67.191287, 56.006194],
                        [-67.244563, 56.866571],
                        [-66.046137, 57.546222],
                        [-65.921232, 57.425878],
                    ]
                ],
                [
                    [
                        [-62.461734, 55.793861],
                        [-61.702232, 55.226953],
                        [-63.246204, 53.479829],
                        [-62.461734, 55.793861],
                    ]
                ],
                [
                    [
                        [-61.702232, 55.226953],
                        [-60.798058, 54.684416],
                        [-62.829759, 53.070215],
                        [-63.143452, 53.185966],
                        [-63.291536, 53.375844],
                        [-63.246204, 53.479829],
                        [-61.702232, 55.226953],
                    ]
                ],
                [
                    [
                        [-63.761308, 51.170749],
                        [-63.758966, 52.381606],
                        [-63.143452, 53.185966],
                        [-62.829759, 53.070215],
                        [-61.540015, 51.510769],
                        [-62.423765, 51.203424],
                        [-63.761308, 51.170749],
                    ]
                ],
                [
                    [
                        [-63.761308, 51.170749],
                        [-64.504725, 50.541144],
                        [-64.743651, 50.815689],
                        [-64.307805, 52.234171],
                        [-63.758966, 52.381606],
                        [-63.761308, 51.170749],
                    ]
                ],
                [
                    [
                        [-64.661564, 50.115186],
                        [-65.790402, 47.926576],
                        [-66.897845, 48.521491],
                        [-66.890978, 48.658947],
                        [-66.660415, 49.378892],
                        [-64.661564, 50.115186],
                    ]
                ],
                [
                    [
                        [-65.790402, 47.926576],
                        [-65.41372, 47.682668],
                        [-65.311425, 47.295289],
                        [-66.563924, 46.16575],
                        [-66.689239, 46.131497],
                        [-67.461655, 48.08959],
                        [-66.897845, 48.521491],
                        [-65.790402, 47.926576],
                    ]
                ],
                [
                    [
                        [-66.009697, 51.80042],
                        [-64.98774, 52.362053],
                        [-64.307805, 52.234171],
                        [-64.743651, 50.815689],
                        [-66.009697, 51.80042],
                    ]
                ],
                [
                    [
                        [-64.39133, 46.008699],
                        [-64.414776, 45.922739],
                        [-65.083804, 45.309838],
                        [-66.563924, 46.16575],
                        [-65.311425, 47.295289],
                        [-64.39133, 46.008699],
                    ]
                ],
                [
                    [
                        [-63.293215, 46.516428],
                        [-62.03623, 46.408914],
                        [-62.02117, 46.392798],
                        [-62.255971, 44.797507],
                        [-62.989139, 44.665362],
                        [-63.624771, 44.843543],
                        [-63.723399, 44.937568],
                        [-64.414776, 45.922739],
                        [-64.39133, 46.008699],
                        [-63.293215, 46.516428],
                    ]
                ],
                [
                    [
                        [-67.244563, 56.866571],
                        [-67.191287, 56.006194],
                        [-67.749682, 56.107668],
                        [-68.370847, 56.712079],
                        [-67.244563, 56.866571],
                    ]
                ],
                [
                    [
                        [-67.862662, 54.865204],
                        [-67.915551, 55.050079],
                        [-67.749682, 56.107668],
                        [-67.191287, 56.006194],
                        [-66.572933, 55.679512],
                        [-66.522429, 55.40378],
                        [-66.767766, 54.679264],
                        [-67.862662, 54.865204],
                    ]
                ],
                [
                    [
                        [-64.414776, 45.922739],
                        [-63.723399, 44.937568],
                        [-64.50239, 43.955688],
                        [-65.664771, 44.409635],
                        [-65.083804, 45.309838],
                        [-64.414776, 45.922739],
                    ]
                ],
                [
                    [
                        [-65.083804, 45.309838],
                        [-65.664771, 44.409635],
                        [-66.192858, 44.031389],
                        [-66.60483, 44.375006],
                        [-66.772422, 44.682886],
                        [-66.981801, 45.788497],
                        [-66.689239, 46.131497],
                        [-66.563924, 46.16575],
                        [-65.083804, 45.309838],
                    ]
                ],
                [
                    [
                        [-67.709376, 42.399187],
                        [-66.229319, 43.585762],
                        [-66.052135, 43.378452],
                        [-66.144297, 41.053141],
                        [-66.186908, 41.023722],
                        [-67.545074, 41.475183],
                        [-67.709376, 42.399187],
                    ]
                ],
                [
                    [
                        [-62.255971, 44.797507],
                        [-62.02117, 46.392798],
                        [-60.0, 45.971991],
                        [-60.0, 44.668956],
                        [-60.26029, 44.493097],
                        [-62.103711, 44.596733],
                        [-62.255971, 44.797507],
                    ]
                ],
                [
                    [
                        [-65.664771, 44.409635],
                        [-64.50239, 43.955688],
                        [-64.048674, 43.570106],
                        [-64.486341, 42.521923],
                        [-65.935304, 43.289411],
                        [-66.052135, 43.378452],
                        [-66.229319, 43.585762],
                        [-66.192858, 44.031389],
                        [-65.664771, 44.409635],
                    ]
                ],
                [
                    [
                        [-66.052135, 43.378452],
                        [-65.935304, 43.289411],
                        [-65.612919, 42.065656],
                        [-65.885626, 41.139652],
                        [-66.144297, 41.053141],
                        [-66.052135, 43.378452],
                    ]
                ],
                [
                    [
                        [-65.885626, 41.139652],
                        [-65.612919, 42.065656],
                        [-64.340982, 41.837472],
                        [-64.713039, 41.271618],
                        [-65.885626, 41.139652],
                    ]
                ],
                [
                    [
                        [-65.612919, 42.065656],
                        [-65.935304, 43.289411],
                        [-64.486341, 42.521923],
                        [-64.21284, 41.996761],
                        [-64.340982, 41.837472],
                        [-65.612919, 42.065656],
                    ]
                ],
                [
                    [
                        [-64.486341, 42.521923],
                        [-64.048674, 43.570106],
                        [-63.806547, 43.665685],
                        [-63.157718, 43.496673],
                        [-63.021562, 43.374746],
                        [-62.55729, 41.874168],
                        [-62.593969, 41.868138],
                        [-64.21284, 41.996761],
                        [-64.486341, 42.521923],
                    ]
                ],
                [
                    [
                        [-62.55729, 41.874168],
                        [-63.021562, 43.374746],
                        [-62.105205, 44.113351],
                        [-61.316855, 42.800556],
                        [-61.782087, 41.657115],
                        [-62.55729, 41.874168],
                    ]
                ],
                [
                    [
                        [-63.624771, 44.843543],
                        [-63.806547, 43.665685],
                        [-64.048674, 43.570106],
                        [-64.50239, 43.955688],
                        [-63.723399, 44.937568],
                        [-63.624771, 44.843543],
                    ]
                ],
                [
                    [
                        [-63.624771, 44.843543],
                        [-62.989139, 44.665362],
                        [-63.157718, 43.496673],
                        [-63.806547, 43.665685],
                        [-63.624771, 44.843543],
                    ]
                ],
                [
                    [
                        [-62.255971, 44.797507],
                        [-62.103711, 44.596733],
                        [-62.105205, 44.113351],
                        [-63.021562, 43.374746],
                        [-63.157718, 43.496673],
                        [-62.989139, 44.665362],
                        [-62.255971, 44.797507],
                    ]
                ],
                [
                    [
                        [-62.103711, 44.596733],
                        [-60.26029, 44.493097],
                        [-61.193774, 42.911554],
                        [-61.316855, 42.800556],
                        [-62.105205, 44.113351],
                        [-62.103711, 44.596733],
                    ]
                ],
                [
                    [
                        [-70.0, 40.005073],
                        [-70.0, 43.085494],
                        [-69.000144, 42.515331],
                        [-68.402835, 42.747442],
                        [-67.709376, 42.399187],
                        [-67.545074, 41.475183],
                        [-66.186908, 41.023722],
                        [-66.097214, 40.0],
                        [-69.993402, 40.0],
                        [-70.0, 40.005073],
                    ]
                ],
                [
                    [
                        [-65.784478, 60.0],
                        [-60.62946, 60.0],
                        [-61.907059, 58.570287],
                        [-62.523439, 58.587777],
                        [-62.930959, 59.990927],
                        [-63.816295, 59.184735],
                        [-65.445288, 59.744365],
                        [-65.784478, 60.0],
                    ]
                ],
                [
                    [
                        [-70.0, 55.857332],
                        [-70.0, 60.0],
                        [-68.637582, 60.0],
                        [-68.425974, 58.836434],
                        [-68.801977, 57.941933],
                        [-68.720108, 57.098347],
                        [-68.593112, 57.006576],
                        [-68.509799, 56.771456],
                        [-68.804662, 56.178409],
                        [-70.0, 55.857332],
                    ]
                ],
                [
                    [
                        [-70.0, 53.53475],
                        [-70.0, 55.730663],
                        [-69.068307, 54.980762],
                        [-68.933613, 54.245603],
                        [-69.018096, 54.100934],
                        [-70.0, 53.53475],
                    ]
                ],
                [
                    [
                        [-60.755079, 54.697224],
                        [-60.700817, 55.29907],
                        [-60.0, 56.393113],
                        [-60.0, 54.644156],
                        [-60.755079, 54.697224],
                    ]
                ],
                [
                    [
                        [-61.200486, 57.981275],
                        [-60.0, 59.554048],
                        [-60.0, 56.661935],
                        [-60.59862, 56.828828],
                        [-60.861104, 57.275846],
                        [-61.176593, 57.41325],
                        [-61.200486, 57.981275],
                    ]
                ],
                [
                    [
                        [-63.526359, 40.0],
                        [-63.464372, 40.68111],
                        [-62.593969, 41.868138],
                        [-62.55729, 41.874168],
                        [-61.782087, 41.657115],
                        [-61.316855, 42.800556],
                        [-61.193774, 42.911554],
                        [-60.26029, 44.493097],
                        [-60.0, 44.668956],
                        [-60.0, 40.203216],
                        [-60.288794, 40.0],
                        [-63.526359, 40.0],
                    ]
                ],
            ],
            "type": "MultiPolygon",
        },
        "properties": {},
        "type": "Feature",
    }
    result2 = voronoi(points)
    assert result2["type"] == "Feature"
